; Déclaration de variables globales
globals [
  particle-types ; liste des types de particules disponibles
]

; Déclaration de variables propres aux tortues
turtles-own [
  stuck?   ; indique si la particule est fixée à l'agrégat
  age      ; âge de la particule (temps écoulé avant agrégation)
  type     ; type de la particule (cuivre, argent, or)
]

; Initialisation du modèle
to setup
  clear-all
  set-default-shape turtles "circle" ; forme des particules
  set particle-types ["cuivre" "argent" "or"] ; types possibles
  setup-attractor                     ; création de l'attracteur
  create-particles initial-particles ; création des particules mobiles
  reset-ticks
end

; Création de la structure initiale d'agrégation selon la forme choisie
to setup-attractor
  if attractor-shape = "point" [
    ; Attracteur en un seul point au centre
    ask patch 0 0 [ sprout 1 [
      set color gray
      set stuck? true
      set age 0
      set type "cuivre"
    ]]
  ]
  if attractor-shape = "line" [
    ; Ligne horizontale au centre
    ask patches with [pycor = 0 and abs(pxcor) < 10] [
      sprout 1 [
        set color gray
        set stuck? true
        set age 0
        set type "cuivre"
      ]
    ]
  ]
  if attractor-shape = "cross" [
    ; Croix au centre
    ask patches with [
      (pxcor = 0 and abs(pycor) < 10) or
      (pycor = 0 and abs(pxcor) < 10)
    ] [
      sprout 1 [
        set color gray
        set stuck? true
        set age 0
        set type "cuivre"
      ]
    ]
  ]
end

; Création des particules mobiles
to create-particles [n]
  create-turtles n [
    setxy random-xcor random-ycor ; position aléatoire dans le monde
    set stuck? false
    set age 0
    set type one-of particle-types ; type aléatoire
    update-color ; appliquer la couleur en fonction du type/âge
  ]
end

; Mise à jour de la couleur d'une particule selon son type et son âge
to update-color
  if type = "cuivre" [ set color scale-color orange age 0 200 ]
  if type = "argent" [ set color scale-color white age 0 200 ]
  if type = "or"     [ set color scale-color yellow age 0 200 ]
end

; Boucle principale du modèle
to go
  ask turtles with [not stuck?] [
    brownian-motion         ; déplacement aléatoire
    set age age + 1         ; incrémentation de l'âge
    check-aggregation       ; vérifie si la particule doit s'agréger
    update-color            ; mise à jour couleur selon âge
  ]
  tick
end

; Mouvement brownien : angle aléatoire et avancement
to brownian-motion
  rt random 360
  fd particle-speed
  ; repositionnement si coincée (bord du monde)
  if not can-move? 0 [ setxy random-xcor random-ycor ]
end

; Vérifie si la particule touche une particule déjà agrégée
to check-aggregation
  let nearby-turtles turtles-on neighbors
  if any? nearby-turtles with [stuck?] [
    set stuck? true ; devient fixée
  ]
end
